import json
from channels.generic.websocket import WebsocketConsumer
from asgiref.sync import async_to_sync
from .utils import get_hour_min
from .models import (
    SupporterModel, 
    UserChatModel,
    ChatModel
)


# route: /ws/<str:type>/chat/<str:username>/
class ChatConsumer(WebsocketConsumer):
    
    def connect(self):

        self.user_id = self.scope['url_route']['kwargs']['username']
        self.type = self.scope['url_route']['kwargs']['type']
        self.user = None

        if self.type == 'supporter':

            self.group_name1 = f'chat_supporter_{self.user_id}'
            self.group_name2 = 'unread_msg_board'

            if SupporterModel.objects.filter(supporter_uid=self.user_id, is_active=True).exists():
                self.user = SupporterModel.objects.get(supporter_uid=self.user_id, is_active=True)

                async_to_sync(self.channel_layer.group_add)(
                    self.group_name1,
                    self.channel_name
                )
                async_to_sync(self.channel_layer.group_add)(
                    self.group_name2,
                    self.channel_name
                )
                self.accept()

        elif self.type == 'client':

            self.group_name = f"chat_client_{self.user_id}"

            if UserChatModel.objects.filter(user_chat_uid=self.user_id, is_blocked=False).exists():

                self.user = UserChatModel.objects.get(user_chat_uid=self.user_id, is_blocked=False)
                async_to_sync(self.channel_layer.group_add)(
                    self.group_name,
                    self.channel_name
                )
                self.accept()


    def disconnect(self, close_code):

        if self.type == 'client':
            async_to_sync(self.channel_layer.group_discard)(
                self.group_name,
                self.channel_name
            )

        elif self.type == 'supporter':
            async_to_sync(self.channel_layer.group_discard)(
                self.group_name1,
                self.channel_name
            )
            async_to_sync(self.channel_layer.group_discard)(
                self.group_name2,
                self.channel_name
            )


    def receive(self, text_data=None, bytes_data=None):

        if text_data and not bytes_data:

            text_data_json = json.loads(text_data)
            # print(self.type.upper(), text_data_json)

            # send delete message to client
            if self.type == 'supporter' and text_data_json.get('_type_request') == 'del':

                chat = ChatModel.objects.get(
                    id=text_data_json.get('id'),
                    sender='supporter'
                )
                chat.is_deleted = True
                chat.save()

                user_group_name = f"chat_client_{text_data_json['client_id']}" 
                async_to_sync(self.channel_layer.group_send)(
                    user_group_name, {
                        'type': 'send_msg',
                        'message': text_data
                    }
                )

            
            # send delete message to supporter
            if self.type == 'client' and text_data_json.get('_type_request') == 'del':

                chat = ChatModel.objects.get(
                    id=text_data_json.get('id'),
                    sender='client'
                )
                chat.is_deleted = True
                chat.save()

                async_to_sync(self.channel_layer.group_send)(
                    'unread_msg_board', {
                        'type': 'send_msg',
                        'message': text_data
                    }
                )

            
            # send edit message to client
            if self.type == 'supporter' and text_data_json.get('_type_request') == 'edit':

                chat = ChatModel.objects.get(
                    id=text_data_json.get('reply_id'),
                    sender='supporter'
                )
                chat.old_msg = chat.msg
                chat.msg = text_data_json.get('text')
                chat.is_edited = True
                chat.save()

                user_group_name = f"chat_client_{text_data_json['client_id']}" 
                async_to_sync(self.channel_layer.group_send)(
                    user_group_name, {
                        'type': 'send_msg',
                        'message': text_data
                    }
                )

            
            # send edit message to supporter
            if self.type == 'client' and text_data_json.get('_type_request') == 'edit':

                chat = ChatModel.objects.get(
                    id=text_data_json.get('reply_id'),
                    sender='client'
                )
                chat.old_msg = chat.msg
                chat.msg = text_data_json.get('text')
                chat.is_edited = True
                chat.save()

                async_to_sync(self.channel_layer.group_send)(
                    'unread_msg_board', {
                        'type': 'send_msg',
                        'message': text_data
                    }
                )

            
            # send status to client
            if self.type == 'supporter' and text_data_json.get('_type_request') == 'status':

                user_group_name = f"chat_client_{text_data_json['client_id']}" 
                async_to_sync(self.channel_layer.group_send)(
                    user_group_name, {
                        'type': 'send_msg',
                        'message': text_data
                    }
                )

            
            # send status to supporter (unread msgs board)
            elif self.type == 'client' and text_data_json.get('_type_request') == 'status': 

                async_to_sync(self.channel_layer.group_send)(
                    'unread_msg_board', {
                        'type': 'send_msg',
                        'message': text_data
                    }
                )

            
            # seen message of client in supporter panel
            elif self.type == 'supporter' and text_data_json.get('_type_request') == 'seen':

                if text_data_json.get('message_sender') == 'client':

                    client = UserChatModel.objects.get(user_chat_uid=text_data_json.get('client_id'))

                    ChatModel.objects.filter(
                        sender='client',
                        client=client,
                        supporter=self.user,
                        is_seen=False
                    ).update(
                        is_seen=True
                    )

                    user_group_name = f"chat_client_{client.user_chat_uid}" 
                    async_to_sync(self.channel_layer.group_send)(
                        user_group_name, {
                            'type': 'send_msg',
                            'message': text_data
                        }
                    )

                
            # seen message of supporter in client
            elif self.type == 'client' and text_data_json.get('_type_request') == 'seen':

                if text_data_json.get('message_sender') == 'supporter':

                    ChatModel.objects.filter(
                        sender='supporter',
                        client=self.user,
                        is_seen=False
                    ).update(
                        is_seen=True
                    )

                    async_to_sync(self.channel_layer.group_send)(
                        'unread_msg_board', {
                            'type': 'send_msg',
                            'message': text_data
                        }
                    )


            # send msg from supporter to client
            elif self.type == 'supporter' and text_data_json.get('_type_request') == 'send':

                user_client = UserChatModel.objects.get(
                    user_chat_uid=text_data_json['client_id'], 
                    is_blocked=False
                )

                chat_obj = ChatModel(
                    supporter=self.user,
                    client=user_client,
                    sender=text_data_json['sender_type'],
                    msg=text_data_json['text']
                )

                if text_data_json['reply_id']:
                    chat_obj.reply = ChatModel.objects.get(id=text_data_json['reply_id'])

                if not user_client.have_supporter:
                    user_client.have_supporter = self.user
                    user_client.save()

                if ChatModel.objects.filter(client=user_client, supporter=None).exists():
                    ChatModel.objects.filter(
                        client=user_client, 
                        supporter=None
                    ).update(
                        supporter=self.user
                    )

                chat_obj.save()

                owner_name_str = f'{self.user.user.first_name} {self.user.user.last_name}' if self.user.user.first_name else 'supporter'
                text_data_json['owner_name'] = owner_name_str
                text_data_json['id'] = chat_obj.id
                text_data_json['created'] = get_hour_min(chat_obj.created)
                text_data_json['is_seen'] = chat_obj.is_seen
                if chat_obj.reply:
                    text_data_json['reply_title'] = chat_obj.reply.sender
                    text_data_json['reply_msg'] = chat_obj.reply.msg
                    text_data_json['reply_id'] = chat_obj.reply.id
                text_data = json.dumps(text_data_json)

                # send msg to client
                user_group_name_1 = f'chat_client_{chat_obj.client.user_chat_uid}' 
                async_to_sync(self.channel_layer.group_send)(
                    user_group_name_1, {
                        'type': 'send_msg',
                        'message': text_data
                    }
                )

                # Echo msg supporter
                user_group_name_2 = 'unread_msg_board'
                async_to_sync(self.channel_layer.group_send)(
                    user_group_name_2, {
                        'type': 'send_msg',
                        'message': text_data
                    }
                )

            
            # send msg from client to supporter
            elif self.type == 'client' and text_data_json.get('_type_request') == 'send':

                chat_obj = ChatModel(
                    client=self.user,
                    sender=text_data_json['sender_type'],
                    msg=text_data_json['text']
                )

                if self.user.have_supporter:
                    chat_obj.supporter = self.user.have_supporter

                if len(text_data_json.get('supporter_uid')) > 1 and not chat_obj.supporter:
                    try:
                        supporter_found = SupporterModel.objects.get(supporter_uid=text_data_json.get('supporter_uid'))
                        chat_obj.supporter = supporter_found
                    except:
                        pass

                if text_data_json['reply_id']:
                    chat_obj.reply = ChatModel.objects.get(
                        id=text_data_json['reply_id']
                    )
                
                chat_obj.save()

                text_data_json['owner_name'] = f'{self.user.first_name} {self.user.last_name}'
                text_data_json['id'] = chat_obj.id
                text_data_json['created'] = get_hour_min(chat_obj.created)
                text_data_json['is_seen'] = chat_obj.is_seen
                if chat_obj.reply:
                    text_data_json['reply_title'] = chat_obj.reply.sender
                    text_data_json['reply_msg'] = chat_obj.reply.msg
                    text_data_json['reply_id'] = chat_obj.reply.id
                if len(text_data_json.get('supporter_uid')) > 1 or self.user.have_supporter:
                    text_data_json['client_have_supporter'] = 'yes'
                else:
                    text_data_json['client_have_supporter'] = 'no'
                text_data = json.dumps(text_data_json)

                # send msg to unread msgs board
                user_group_name_1 = "unread_msg_board"    
                async_to_sync(self.channel_layer.group_send)(
                    user_group_name_1, {
                        'type': 'send_msg',
                        'message': text_data
                    }
                )
                # Echo msg client
                user_group_name_2 = f"chat_client_{self.user_id}"    
                async_to_sync(self.channel_layer.group_send)(
                    user_group_name_2, {
                        'type': 'send_msg',
                        'message': text_data
                    }
                )

            # send supporterID to client
            elif self.type == 'supporter' and text_data_json.get('_type_request') == 'set_supporter_for_client':

                try:
                    supporter_found = SupporterModel.objects.get(supporter_uid=text_data_json.get('supporter_id'))
                    client_found = UserChatModel.objects.get(user_chat_uid=text_data_json.get('client_id'))
                    client_found.have_supporter = supporter_found
                    client_found.save()
                except:
                    pass

                # Echo msg client
                user_group_name = f"chat_client_{text_data_json.get('client_id')}"    
                async_to_sync(self.channel_layer.group_send)(
                    user_group_name, {
                        'type': 'send_msg',
                        'message': text_data
                    }
                )

    

    def send_msg(self, event):
        message = event['message']
        self.send(text_data=message)





from django.contrib import admin
from django.urls import path, include
from chatapp.views import supporter_homepage


urlpatterns = [
    path('admin/', admin.site.urls),
    path('', include('test_1.urls')),
    path('django-chatapp/chat/supporter/', supporter_homepage),
    path('', include('chatapp.urls', namespace='chatapp')),
]





from pathlib import Path


BASE_DIR = Path(__file__).resolve().parent.parent
SECRET_KEY = 'django-insecure-r^p0kgfm$szo5=dk1y$x)w@21kc_4y(n7_@%ogged7o27_1or@'
DEBUG = True
ALLOWED_HOSTS = []


# Application definition
INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',

    'daphne',
    'channels',
    'django.contrib.staticfiles',
    'chatapp',
    'test_1',

]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'App_Config.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [ Path('templates') ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]


ASGI_APPLICATION = 'App_Config.asgi.application'
CHANNEL_LAYERS = {
    "default": {
        "BACKEND": "channels.layers.InMemoryChannelLayer"
    }
}

# Database
# https://docs.djangoproject.com/en/4.1/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': BASE_DIR / 'db.sqlite3',
    }
}


# Password validation
# https://docs.djangoproject.com/en/4.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/4.1/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_TZ = True

STATIC_URL = '/site_static/'
STATIC_ROOT = Path("static_cdn", "static_root")
MEDIA_URL = '/media/'
MEDIA_ROOT = Path("static_cdn", "media_root")
STATICFILES_DIRS = [ Path("assets") ]

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'


# ### DJANGO CHAT APP CONFIGS ### #
CHATAPP_DIR = 'ltr'
CHATAPP_AUTHFIELDS = 'phone' # or 'phone'



import os
from django.core.asgi import get_asgi_application
from channels.routing import ProtocolTypeRouter, URLRouter
from channels.auth import AuthMiddlewareStack
from channels.sessions import SessionMiddlewareStack
from chatapp.routing import ws_urlpatterns


os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'App_Config.settings')

application = ProtocolTypeRouter({
    'http': get_asgi_application(),
    'websocket': SessionMiddlewareStack(
        AuthMiddlewareStack(
            URLRouter(ws_urlpatterns)
        )
    )
})


# Generated by Django 4.1.7 on 2023-03-23 09:21

import chatapp.utils
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='SupporterModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('supporter_uid', models.CharField(default=chatapp.utils.unique_username, editable=False, max_length=255, unique=True, verbose_name='آیدی پشتیبان')),
                ('full_name', models.CharField(blank=True, max_length=255, null=True, verbose_name='نام نمایشی')),
                ('is_active', models.BooleanField(default=False, verbose_name='فعال / غیرفعال')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='کاربر')),
            ],
            options={
                'verbose_name': 'پشتیبان',
                'verbose_name_plural': 'پشتیبان ها',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='UserChatModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('user_chat_uid', models.CharField(default=chatapp.utils.unique_username, editable=False, max_length=255, unique=True, verbose_name='آیدی چت کاربر')),
                ('first_name', models.CharField(max_length=255, verbose_name='نام')),
                ('last_name', models.CharField(max_length=255, verbose_name='نام خانوادگی')),
                ('email', models.CharField(blank=True, max_length=255, null=True, verbose_name='ایمیل')),
                ('phone', models.CharField(blank=True, max_length=255, null=True, verbose_name='شماره تماس')),
                ('is_blocked', models.BooleanField(default=False, verbose_name='آیا مسدود شده است؟')),
                ('report_numbers', models.IntegerField(default=0, verbose_name='تعداد گزارش ها')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('have_supporter', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='chatapp.supportermodel', verbose_name='آیا پشتیبان دارد؟')),
            ],
            options={
                'verbose_name': 'اطلاعات کاربر',
                'verbose_name_plural': 'اطلاعات کاربران',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='ReportUserModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('item', models.CharField(choices=[('badterms', 'استفاده از کلمات نامناسب'), ('others', 'دلایل دیگر')], max_length=12, verbose_name='دلیل گزارش')),
                ('desc', models.TextField(verbose_name='متن گزارش')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('supporter', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='chatapp.supportermodel', verbose_name='پشتیبان')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='chatapp.userchatmodel', verbose_name='کاربر')),
            ],
            options={
                'verbose_name': 'گزارش کاربر',
                'verbose_name_plural': 'گزارش کاربران',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='ReadyChatModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('subject', models.CharField(max_length=255, verbose_name='عنوان')),
                ('content', models.TextField(verbose_name='محتوا')),
                ('is_public', models.BooleanField(default=False, verbose_name='آیا برای تمامی پشتیبان ها نمایش داده شود؟')),
                ('supporter', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='chatapp.supportermodel', verbose_name='پشتیبان')),
            ],
            options={
                'verbose_name': 'پیام آماده ی پشتیبان',
                'verbose_name_plural': 'پیام های آماده ی پشتیبان ها',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='ChatModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sender', models.CharField(choices=[('supporter', 'پشتیبان'), ('client', 'کاربر')], max_length=255, verbose_name='ارسال کننده')),
                ('msg', models.TextField(verbose_name='متن پیام')),
                ('is_seen', models.BooleanField(default=False, verbose_name='آیا این پیام دیده شده است؟')),
                ('is_edited', models.BooleanField(default=False, verbose_name='آیا این پیام ویرایش شده است؟')),
                ('old_msg', models.TextField(blank=True, null=True, verbose_name='متن پیام قبل از ویرایش')),
                ('is_deleted', models.BooleanField(default=False, verbose_name='آیا این پیام حذف شده است؟')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('client', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='chatapp.userchatmodel', verbose_name='کاربر')),
                ('reply', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='chatapp.chatmodel', verbose_name='پاسخ')),
                ('supporter', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='chatapp.supportermodel', verbose_name='پشتیبان')),
            ],
            options={
                'verbose_name': 'چت کاربر و پشتیبان',
                'verbose_name_plural': 'چت های کاربر و پشتیبان',
                'ordering': ['-id'],
            },
        ),
    ]



import re
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.utils import timezone
from random import randint


def is_email(email):
    regex = r'(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$)'
    return bool(re.compile(regex).match(str(email)))


def is_phone(phone):
    regex = r'0?9([0-1][0-9]|3[1-9]|2[1-9])-?[0-9]{3}-?[0-9]{4}'
    return bool(re.compile(regex).match(str(phone)))


def get_hour_min(created):
    """ usage: created time of messages in chat box """

    h = timezone.localtime(created).hour
    m = timezone.localtime(created).minute

    if h <= 9:
        h = f'0{h}'
    if m <= 9:
        m = f'0{m}'

    return f'{h}:{m}'


def unique_username():
    """ usage: set username for supporter or client """
    
    time = timezone.now()
    year_str = str(time.year)

    random = randint(1000, 9999)
    month = time.month
    day = time.day
    
    if month < 10:
        month = f'0{month}'
    if day < 10:
        day = f'0{day}'
    
    return f'{year_str[-2:]}{month}{day}{random}' 
    # sample: output is 2303174327 => 23: year / 03: month / 17: day / 4327: random


def get_settings():

    setting_dict = {}
            
    try:
        setting_dict['dir'] = settings.CHATAPP_DIR
    except:
        setting_dict['dir'] = 'ltr'
    
    try:
        setting_dict['title'] = settings.CHATAPP_TITLE
    except:
        setting_dict['title'] = _('خوش آمدید')
    
    try:
        setting_dict['subtitle'] = settings.CHATAPP_SUBTITLE
    except:
        setting_dict['subtitle'] = _('لطفا کمی صبر کنید')
    
    try:
        setting_dict['game'] = settings.CHATAPP_GAME
    except:
        setting_dict['game'] = True
    
    try:
        setting_dict['auth_fields'] = settings.CHATAPP_AUTHFIELDS
    except:
        setting_dict['auth_fields'] = 'email'
    
    try:
        setting_dict['max_report_number'] = settings.CHATAPP_MAX_REPORT_NUMBER
    except:
        setting_dict['max_report_number'] = 2
    
    try:
        setting_dict['edit_user_msg'] = settings.CHATAPP_EDIT_USER_MESSAGE
    except:
        setting_dict['edit_user_msg'] = True
    
    try:
        setting_dict['edit_supporter_msg'] = settings.CHATAPP_EDIT_SUPPORTER_MESSAGE
    except:
        setting_dict['edit_supporter_msg'] = True
    
    try:
        setting_dict['delete_user_msg'] = settings.CHATAPP_DELETE_USER_MESSAGE
    except:
        setting_dict['delete_user_msg'] = True
    
    try:
        setting_dict['delete_supporter_msg'] = settings.CHATAPP_DELETE_SUPPORTER_MESSAGE
    except:
        setting_dict['delete_supporter_msg'] = True
    
    try:
        setting_dict['show_deleted_msg'] = settings.CHATAPP_SHOW_DELETED_MESSAGE
    except:
        setting_dict['show_deleted_msg'] = True

    try:
        setting_dict['language'] = settings.CHATAPP_LANGUAGE
    except:
        setting_dict['language'] = 'en'

    return setting_dict




from django.contrib import admin
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from .models import (
    SupporterModel,
    ChatModel,
    UserChatModel,
    ReadyChatModel,
    ReportUserModel
)


# write actions for UserChatModel
@admin.action(description=_('حذف کاربران غیرفعال در ۳۰ روز اخیر'))
def del_users_more_30d(modeladmin, request, queryset):
    onemonth_ago = timezone.now() - timezone.timedelta(days=30)
    UserChatModel.objects.filter(
        created__lt=onemonth_ago
    ).delete()


@admin.action(description=_('حذف کاربرانی که مسدود شده اند'))
def del_users_isblocked(modeladmin, request, queryset):
    UserChatModel.objects.filter(
        is_blocked=True
    ).delete()


@admin.action(description=_('آزادکردن کاربرانی که مسدود شده اند'))
def unblock_users(modeladmin, request, queryset):
    UserChatModel.objects.filter(
        is_blocked=True
    ).update(
        is_blocked=False,
        report_numbers=0
    )


# write admin models
class SupporterModel_Admin(admin.ModelAdmin):
    list_display = ['user', 'supporter_uid', 'full_name', 'is_active']
    search_field = ['supporter_uid']
    ordering = ['-id']

    
class ChatModel_Admin(admin.ModelAdmin):
    list_display = ['id', 'client', 'supporter', 'sender', 'is_seen']
    ordering = ['-id']

    
class UserChatModel_Admin(admin.ModelAdmin):
    list_display = ['user_chat_uid', 'have_supporter', 'created', 'is_blocked']
    search_field = ['user_chat_uid']
    ordering = ['-id']
    actions = [del_users_more_30d, del_users_isblocked, unblock_users]

    
class ReadyChatModel_Admin(admin.ModelAdmin):
    list_display = ['subject', 'supporter', 'is_public']
    search_field = ['subject']
    ordering = ['-id']
    

class ReportUserModel_Admin(admin.ModelAdmin):
    list_display = ['user', 'item', 'created']
    ordering = ['-id']


admin.site.register(SupporterModel, SupporterModel_Admin)
admin.site.register(ChatModel, ChatModel_Admin)
admin.site.register(UserChatModel, UserChatModel_Admin)
admin.site.register(ReadyChatModel, ReadyChatModel_Admin)
admin.site.register(ReportUserModel, ReportUserModel_Admin)



from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import gettext_lazy as _
from .utils import unique_username


class SupporterModel(models.Model):
    """ for creating a list of supporter for your website """

    supporter_uid = models.CharField(default=unique_username, max_length=255, editable=False, unique=True, verbose_name=_('آیدی پشتیبان'))
    user = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_('کاربر'))
    full_name = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('نام نمایشی'))
    is_active = models.BooleanField(default=False, verbose_name=_('فعال / غیرفعال'))
    created = models.DateTimeField(auto_now_add=True)
    
    class Meta:
        ordering = ['-id']
        verbose_name = _('پشتیبان')
        verbose_name_plural = _('پشتیبان ها')
    
    def __str__(self):
        try:
            return f'{self.full_name} ({str(self.supporter_uid)})'
        except:
            return f'{str(self.supporter_uid)}'


class ChatModel(models.Model):
    """ this is a message data between client and supporter """
    
    CLIENT_SUPPORTER = (('supporter', _('پشتیبان')), ('client', _('کاربر')))

    reply = models.ForeignKey('self', on_delete=models.CASCADE, blank=True, null=True, verbose_name=_('پاسخ'))
    client = models.ForeignKey('UserChatModel', on_delete=models.CASCADE, verbose_name=_('کاربر'))
    supporter = models.ForeignKey(SupporterModel, blank=True, null=True, on_delete=models.CASCADE, verbose_name=_('پشتیبان'))
    sender = models.CharField(max_length=255, choices=CLIENT_SUPPORTER, verbose_name=_('ارسال کننده'))
    msg = models.TextField(verbose_name=_('متن پیام'))
    is_seen = models.BooleanField(default=False, verbose_name=_('آیا این پیام دیده شده است؟'))
    is_edited = models.BooleanField(default=False, verbose_name=_('آیا این پیام ویرایش شده است؟'))
    old_msg = models.TextField(blank=True, null=True, verbose_name=_('متن پیام قبل از ویرایش'))
    is_deleted = models.BooleanField(default=False, verbose_name=_('آیا این پیام حذف شده است؟'))
    created = models.DateTimeField(auto_now_add=True)
    
    class Meta:
        ordering = ['-id']
        verbose_name = _('چت کاربر و پشتیبان')
        verbose_name_plural = _('چت های کاربر و پشتیبان')
    
    def __str__(self):
        return str(self.id)


class UserChatModel(models.Model):
    """ create a flag for user for chat to supporter """

    user_chat_uid = models.CharField(default=unique_username, max_length=255, editable=False, unique=True, verbose_name=_('آیدی چت کاربر'))
    have_supporter = models.ForeignKey(SupporterModel, on_delete=models.CASCADE, blank=True, null=True, verbose_name=_('آیا پشتیبان دارد؟'))
    first_name = models.CharField(max_length=255, verbose_name=_('نام'))
    last_name = models.CharField(max_length=255, verbose_name=_('نام خانوادگی'))
    email = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('ایمیل'))
    phone = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('شماره تماس'))
    is_blocked = models.BooleanField(default=False, verbose_name=_('آیا مسدود شده است؟'))
    report_numbers = models.IntegerField(default=0, verbose_name=_('تعداد گزارش ها'))
    created = models.DateTimeField(auto_now_add=True)

    class Meta:
        ordering = ['-id']
        verbose_name = _('اطلاعات کاربر')
        verbose_name_plural = _('اطلاعات کاربران')
    
    def __str__(self):
        return str(self.user_chat_uid)


class ReadyChatModel(models.Model):
    """ list of ready chat component for all of supporters """

    subject = models.CharField(max_length=255, verbose_name=_('عنوان'))
    content = models.TextField(verbose_name=_('محتوا'))
    supporter = models.ForeignKey(SupporterModel, on_delete=models.CASCADE, blank=True, null=True, verbose_name=_('پشتیبان'))
    is_public = models.BooleanField(default=False, verbose_name=_('آیا برای تمامی پشتیبان ها نمایش داده شود؟'))

    class Meta:
        ordering = ['-id']
        verbose_name = _('پیام آماده ی پشتیبان')
        verbose_name_plural = _('پیام های آماده ی پشتیبان ها')
    
    def __str__(self):
        return self.subject


class ReportUserModel(models.Model):
    """ report users from supporters is here """

    REPORT_ITEMS = (('badterms', _('استفاده از کلمات نامناسب')), ('others', _('دلایل دیگر')))
    supporter = models.ForeignKey(SupporterModel, on_delete=models.CASCADE, verbose_name=_('پشتیبان'))
    user = models.ForeignKey(UserChatModel, on_delete=models.CASCADE, verbose_name=_('کاربر'))
    item = models.CharField(max_length=12, choices=REPORT_ITEMS, verbose_name=_('دلیل گزارش'))
    desc = models.TextField(verbose_name=_('متن گزارش'))
    created = models.DateTimeField(auto_now_add=True)

    class Meta:
        ordering = ['-id']
        verbose_name = _('گزارش کاربر')
        verbose_name_plural = _('گزارش کاربران')
    
    def __str__(self):
        return self.user.user_chat_uid
    



from django.urls import path
from . import views


app_name = 'chatapp'

urlpatterns = [
    path('django-chatapp/chat/supporter/login-required/', views.loginrequired_page, name='login-required'),
    path('django-chatapp/chat/supporter/', views.supporter_homepage, name='sup-homepage'),
    path('django-chatapp/chat/supporter/unreads/', views.supporter_unreads, name='sup-unreads'),
    path('django-chatapp/chat/supporter/read-all/', views.supporter_read_all, name='sup-readall'),
    path('django-chatapp/chat/supporter/ready-msg/get/', views.get_ready_msg, name='sup-getreadymsg'),
    path('django-chatapp/chat/supporter/ready-msg/del/', views.delete_ready_msg, name='sup-delreadymsg'),
    path('django-chatapp/chat/supporter/ready-msg/create/', views.create_ready_msg, name='sup-createreadymsg'),
    path('django-chatapp/chat/supporter/report/', views.report_user, name='sup-report'),

    path('django-chatapp/auth/check/userid/', views.check_userid, name='check-userid'),
    path('django-chatapp/auth/create/userid/', views.create_userid, name='create-userid'),
    path('django-chatapp/chat/setting/', views.setting_chat, name='setting-chat'),
]


from django.db.models import Q
from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.views.decorators.csrf import csrf_exempt
from django.http import JsonResponse, HttpResponse
from django.shortcuts import render, get_object_or_404, redirect
from .utils import get_hour_min, get_settings
from .models import (
    SupporterModel,
    UserChatModel,
    ChatModel,
    ReadyChatModel,
    ReportUserModel
)


def ready_message_list(supporter) -> list:
    """ get list of ready message of a supporter """

    lookup = Q(supporter=supporter) | Q(is_public=True)
    ready_chats = ReadyChatModel.objects.filter(lookup).all()

    msgs_list = list(ready_chats.values(
        'id', 'subject', 'content', 'supporter__supporter_uid', 'is_public'
    ))
    return msgs_list


# url: /django-chatapp/chat/supporter/login-required/
def loginrequired_page(request):
    """ go to supporter login page OR show h3 html tag """

    try:
        login_url = settings.CHATAPP_SUPPORTER_LOGIN_URL
        return redirect(login_url)
    except:
        login_url = None
        return HttpResponse('<h3>Login Required.</h3>')


# url: /django-chatapp/auth/check/userid/
@csrf_exempt
def check_userid(request):
    """ 
    - send chat settings
    - check userID and send messages to her chat app 
    - seen messages of supporter via this user 
    """

    if request.method == 'POST':

        userid = request.POST.get('user_id')
        
        # send 10 msg from db to client
        if userid and UserChatModel.objects.filter(
            user_chat_uid=userid, 
            is_blocked=False
        ).exists():
            
            try:
                msg_counter = settings.CHATAPP_MESSAGES_COUNT
            except:
                msg_counter = 30

            client = UserChatModel.objects.get(
                user_chat_uid=userid, 
                is_blocked=False
            )
            chats = ChatModel.objects.filter(client=client).all()[:msg_counter]
            chats_arr = []

            for item in chats:
                obj = {
                    'id': item.id,
                    'owner_id': item.supporter.supporter_uid if item.sender == 'supporter' else item.client.user_chat_uid,
                    'owner_name': 'supporter' if item.sender == 'supporter' else f'{item.client.first_name} {item.client.last_name}',
                    'sender_type': item.sender,
                    'reply_id': item.reply.id if item.reply else '',
                    'reply_title': item.reply.sender if item.reply else '',
                    'reply_msg': item.reply.msg if item.reply else '',
                    'reply_is_deleted': True if item.reply and item.reply.is_deleted else False,
                    'is_seen': item.is_seen,
                    'is_edited': item.is_edited,
                    'is_deleted': item.is_deleted,
                    'created': get_hour_min(item.created),
                    'text': item.msg
                }
                chats_arr.append(obj)

            # update the unread messages
            ChatModel.objects.filter(
                sender='supporter',
                is_seen=False,
                client=client
            ).update(is_seen=True)

            chats_arr.reverse()

            return JsonResponse({
                'data': chats_arr, 
                'setting': get_settings(),
                'status': 200
            })
        
        return JsonResponse({'status': 401})
    return JsonResponse({'status': 400})


# url: /django-chatapp/auth/create/userid/
@csrf_exempt
def create_userid(request):
    """ create a user with fname, lname, email or phone. this is client login from! """

    if request.method == 'POST':
        fname = request.POST.get('fname')
        lname = request.POST.get('lname')
        email = str()
        phone = str()

        if request.POST.get('email'):
            email = request.POST.get('email')
        elif request.POST.get('phone'):
            phone = request.POST.get('phone')

        user_chat_model = UserChatModel(
            first_name=fname,
            last_name=lname
        )

        if email:
            user_chat_model.email = email
        elif phone:
            user_chat_model.phone = phone

        user_chat_model.save()

        return JsonResponse({
            'data': user_chat_model.user_chat_uid, 
            'status': 200
        })
    
    return JsonResponse({'status': 400})


# url: /django-chatapp/chat/setting/
@csrf_exempt
def setting_chat(request):
    """ send chat settings """


    if request.method == 'POST':
        return JsonResponse({
            'data': get_settings(), 
            'status': 200
        })
    
    return JsonResponse({'status': 400})


# url: /django-chatapp/chat/supporter/
@login_required(login_url='chatapp:login-required')
def supporter_homepage(request):
    """ check supporter in db and show panel """

    try:
        supporter_uid = SupporterModel.objects.get(
            user=request.user, 
            is_active=True
        )
        return render(request, 'supporter.html', {
            'supporter_uid': supporter_uid.supporter_uid
        })
    except:
        return HttpResponse('<h3>You are not a supporter!</h3>')


# url: /django-chatapp/chat/supporter/unreads/
@login_required(login_url='chatapp:login-required')
@csrf_exempt
def supporter_unreads(request):
    """ 
    send unread messages for this supporter
    send unread messages that do not have supporter
    send all clients 
    """
    
    if request.method == 'POST':

        supporter_uid = request.POST.get('supporter_uid')
        
        if not supporter_uid or not SupporterModel.objects.filter(
            supporter_uid=supporter_uid, 
            is_active=True
        ).exists():
            return HttpResponse('<h3>You are not a supporter!</h3>')
        
        chats__nosupporter = ChatModel.objects.filter(
            sender='client',
            is_seen=False,
            supporter=None
        ).order_by('id')

        chats__thissupporter = ChatModel.objects.filter(
            sender='client',
            is_seen=False,
            supporter__user=request.user
        ).order_by('id')

        unreads_nosupoorter = []
        unreads_thissupporter = []

        for item in chats__nosupporter:

            obj = {
                'id': item.id,
                'owner_id': item.supporter.supporter_uid if item.sender == 'supporter' else item.client.user_chat_uid,
                'owner_name': 'supporter' if item.sender == 'supporter' else f'{item.client.first_name} {item.client.last_name}',
                'sender_type': item.sender,
                'reply_id': item.reply.id if item.reply else '',
                'reply_title': item.reply.sender if item.reply else '',
                'reply_is_deleted': True if item.reply and item.reply.is_deleted else False,
                'reply_msg': item.reply.msg if item.reply else '',
                'is_seen': item.is_seen,
                'is_edited': item.is_edited,
                'is_deleted': item.is_deleted,
                'created': get_hour_min(item.created),
                'text': item.msg
            }
            unreads_nosupoorter.append(obj)

        for item in chats__thissupporter:

            obj = {
                'id': item.id,
                'owner_id': item.supporter.supporter_uid if item.sender == 'supporter' else item.client.user_chat_uid,
                'owner_name': 'supporter' if item.sender == 'supporter' else f'{item.client.first_name} {item.client.last_name}',
                'sender_type': item.sender,
                'reply_id': item.reply.id if item.reply else '',
                'reply_title': item.reply.sender if item.reply else '',
                'reply_is_deleted': True if item.reply and item.reply.is_deleted else False,
                'reply_msg': item.reply.msg if item.reply else '',
                'is_seen': item.is_seen,
                'is_edited': item.is_edited,
                'is_deleted': item.is_deleted,
                'created': get_hour_min(item.created),
                'text': item.msg
            }
            unreads_thissupporter.append(obj)

        chat_to_all = list(
            UserChatModel.objects.filter(
                is_blocked=False
            ).values(
                'user_chat_uid', 'first_name', 'last_name'
            )
        )

        return JsonResponse({
            'unreads_nosupoorter': unreads_nosupoorter, 
            'unreads_thissupporter': unreads_thissupporter, 
            'chat_to_all': chat_to_all,
            'status': 200
        })

    return JsonResponse({'status': 400})


# url: /django-chatapp/chat/supporter/read-all/
@login_required(login_url='chatapp:login-required')
@csrf_exempt
def supporter_read_all(request):
    """ 
    send chats between client and supporter
    set supporter for client
    seen messages of client for this supporter
    """

    if request.method == 'POST':

        supporter_uid = request.POST.get('supporter_uid')
        client_id= request.POST.get('client_id')
        msg_type= request.POST.get('msg_type')
        
        if not client_id or not msg_type or not SupporterModel.objects.filter(
            supporter_uid=supporter_uid, 
            is_active=True
        ).exists():
            return HttpResponse('<h3>You are not a supporter!</h3>')
        
        supporter = SupporterModel.objects.get(user=request.user, is_active=True)
        client = UserChatModel.objects.get(user_chat_uid=client_id, is_blocked=False)
        
        if msg_type == 'thissupporter':

            # update the unread messages
            ChatModel.objects.filter(
                sender='client',
                is_seen=False,
                supporter=supporter,
                client=client
            ).update(
                is_seen=True
            )
    
        elif msg_type == 'nosupoorter':

            # add supporter for this client
            UserChatModel.objects.filter(
                user_chat_uid=client_id,
                have_supporter=None
            ).update(
                have_supporter=supporter
            )

            # update the unread messages
            ChatModel.objects.filter(
                sender='client',
                is_seen=False,
                client=client,
                supporter=None
            ).update(
                supporter=supporter,
                is_seen=True
            )

        try:
            msg_counter = settings.CHATAPP_MESSAGES_COUNT
        except:
            msg_counter = 30

        chats = ChatModel.objects.filter(
            supporter=supporter,
            client=client
        ).all()[:msg_counter]
        chats_arr = []

        for item in chats:

            obj = {
                'id': item.id,
                'owner_id': item.supporter.supporter_uid if item.sender == 'supporter' else item.client.user_chat_uid,
                'owner_name': 'supporter' if item.sender == 'supporter' else f'{item.client.first_name} {item.client.last_name}',
                'sender_type': item.sender,
                'reply_id': item.reply.id if item.reply else '',
                'reply_title': item.reply.sender if item.reply else '',
                'reply_is_deleted': True if item.reply and item.reply.is_deleted else False,
                'reply_msg': item.reply.msg if item.reply else '',
                'is_seen': item.is_seen,
                'is_edited': item.is_edited,
                'is_deleted': item.is_deleted,
                'created': get_hour_min(item.created),
                'text': item.msg
            }
            chats_arr.append(obj)
        
        chats_arr.reverse()

        return JsonResponse({
            'data': chats_arr, 
            'status': 200
        })
    
    return JsonResponse({'status': 400})


# url: /django-chatapp/chat/supporter/report/
@login_required(login_url='chatapp:login-required')
@csrf_exempt
def report_user(request):
    """ report a client via supporter """

    if request.method == 'POST':

        supporter_uid = request.POST.get('supporter_uid')
        client_id = request.POST.get('client_id')
        report_cause = request.POST.get('report_cause')
        report_item = request.POST.get('report_item')
    
        if supporter_uid and client_id and report_cause and report_item:

            user = get_object_or_404(UserChatModel, user_chat_uid=client_id)
            supporter = get_object_or_404(SupporterModel, supporter_uid=supporter_uid, is_active=True)

            ReportUserModel.objects.create(
                supporter=supporter,
                user=user,
                item=report_item,
                desc=report_cause
            )

            user.report_numbers = user.report_numbers + 1
            user.save()

            try:
                max_report_num = settings.CHATAPP_MAX_REPORT_NUMBER
            except:
                max_report_num = 2

            if user.report_numbers == int(max_report_num):
                user.is_blocked = True
                user.save()
                return JsonResponse({
                    'status': 200, 
                    'is_blocked': True
                })
            
            return JsonResponse({'status': 200, 'is_blocked': False})
        
        return JsonResponse({'status': 400}) 
    return JsonResponse({'status': 400}) 
                

# url: /django-chatapp/chat/supporter/ready-msg/get/
@login_required(login_url='chatapp:login-required')
@csrf_exempt
def get_ready_msg(request):
    """ send list of ready message to supporter panel """

    if request.method == 'POST':
        
        try:
            supporter_uid = request.POST.get('supporter_uid')
            supporter = SupporterModel.objects.get(supporter_uid=supporter_uid, is_active=True)
        except:
            return HttpResponse('<h3>You are not a supporter!</h3>')

        return JsonResponse({
            'data': ready_message_list(supporter=supporter), 
            'status': 200
        })
    return JsonResponse({'status': 400}) 

            
# url: /django-chatapp/chat/supporter/ready-msg/del/
@login_required(login_url='chatapp:login-required')
@csrf_exempt
def delete_ready_msg(request):
    """ 
    delete ready message of supporter and 
    send list of ready message to supporter panel 
    """

    if request.method == 'POST':

        try:
            supporter_uid = request.POST.get('supporter_uid')
            supporter = SupporterModel.objects.get(supporter_uid=supporter_uid, is_active=True)
        except:
            return HttpResponse('<h3>You are not a supporter!</h3>')

        # delete ready message of supporter
        ReadyChatModel.objects.get(
            id=request.POST.get('msgID'),
            supporter=supporter
        ).delete()

        return JsonResponse({
            'data': ready_message_list(supporter=supporter), 
            'status': 200
        })
    return JsonResponse({'status': 400}) 


# url: /django-chatapp/chat/supporter/ready-msg/create/
@login_required(login_url='chatapp:login-required')
@csrf_exempt
def create_ready_msg(request):
    """ 
    create a ready message via supporter and 
    send list of ready message to supporter panel 
    """

    if request.method == 'POST':

        try:
            subject = request.POST.get('subject')
            content = request.POST.get('content')
            supporter_uid = request.POST.get('supporter_uid')
            supporter = SupporterModel.objects.get(supporter_uid=supporter_uid, is_active=True)
        except:
            return HttpResponse('<h3>You are not a supporter!</h3>')

        ReadyChatModel.objects.create(
            subject=subject,
            content=content,
            supporter=supporter
        )

        return JsonResponse({
            'data': ready_message_list(supporter=supporter), 
            'status': 200
        })
    return JsonResponse({'status': 400}) 




#!/usr/bin/env python
import codecs
from setuptools import setup, find_packages


def read_me(filename):
    return codecs.open(filename, encoding='utf-8').read()


setup(
    name='django-chatapp',
    version='2.1',
    python_requires='<=3.12',
    packages=find_packages(),
    include_package_data=True,
    description=(
        'A flexible Chat Application for open source software society.'
    ),
    url='https://github.com/grand-kaveh/django-chat-app',
    download_url='https://pypi.org/project/django-chatapp/2.1/',
    author='Grand Kaveh',
    author_email='great.kaveh.2000@gmail.com',
    keywords="django chat websocket channels asgi vuejs",
    license='MIT',
    platforms=['any'],
    install_requires=[
        "django",
        "channels",
        "daphne",
    ],
    long_description=read_me('README.md'),
    long_description_content_type='text/markdown',
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)